"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.TestScheduler = void 0;
const events_1 = require("events");
const TestSessionStatus_1 = require("./TestSessionStatus");
class TestScheduler extends events_1.EventEmitter {
    constructor(config, browsers, sessions) {
        super();
        this.config = config;
        this.browsers = browsers;
        this.sessions = sessions;
    }
    async schedule(testRun, sessionsToSchedule) {
        for (const session of sessionsToSchedule) {
            this.sessions.update(Object.assign(Object.assign({}, session), { status: TestSessionStatus_1.STATUS_SCHEDULED }));
        }
        return this.runScheduled(testRun);
    }
    runScheduled(testRun) {
        const scheduleTasks = [];
        const scheduledIt = this.sessions.forStatus(TestSessionStatus_1.STATUS_SCHEDULED);
        const runningCount = Array.from(this.sessions.forStatus(TestSessionStatus_1.STATUS_INITIALIZING, TestSessionStatus_1.STATUS_STARTED))
            .length;
        const count = this.config.concurrency - runningCount;
        for (let i = 0; i < count; i += 1) {
            const { done, value } = scheduledIt.next();
            if (done || !value) {
                break;
            }
            scheduleTasks.push(this.runSession(testRun, value));
        }
        return Promise.all(scheduleTasks);
    }
    runSession(testRun, session) {
        this.sessions.update(Object.assign(Object.assign({}, session), { testRun, status: TestSessionStatus_1.STATUS_INITIALIZING }));
        setTimeout(() => {
            const upToDateSession = this.sessions.get(session.id);
            if (upToDateSession.testRun !== testRun) {
                // session reloaded in the meantime
                return;
            }
            if (upToDateSession.status === TestSessionStatus_1.STATUS_INITIALIZING) {
                this.setSessionTimedout(upToDateSession, `Did not receive a start signal from browser after ${this.config.sessionStartTimeout}ms.`);
                return;
            }
            if (upToDateSession.status === TestSessionStatus_1.STATUS_FINISHED) {
                // The session finished by now
                return;
            }
            setTimeout(() => {
                const upToDateSession = this.sessions.get(session.id);
                if (upToDateSession.testRun !== testRun) {
                    // session reloaded in the meantime
                    return;
                }
                if (upToDateSession.status !== TestSessionStatus_1.STATUS_FINISHED) {
                    this.setSessionTimedout(upToDateSession, `Browser did not finish within ${this.config.sessionStartTimeout}ms.`);
                }
            }, this.config.sessionFinishTimeout);
        }, this.config.sessionStartTimeout);
        // TODO: Select associated browser
        return this.browsers[0].startSession(session);
    }
    setSessionTimedout(session, message) {
        this.emit('session-timed-out', {
            id: session.id,
            result: {
                passed: false,
                logs: [],
                tests: [],
                failedImports: [],
                request404s: new Set(),
                error: { message },
            },
        });
    }
}
exports.TestScheduler = TestScheduler;
//# sourceMappingURL=TestSessionScheduler.js.map