"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.dependencyGraphMiddleware = void 0;
const dependency_graph_1 = require("dependency-graph");
const debounce_1 = __importDefault(require("debounce"));
const path_1 = __importDefault(require("path"));
const constants_1 = require("../../core/constants");
function toFilePath(browserPath) {
    return browserPath.replace(new RegExp('/', 'g'), path_1.default.sep);
}
function dependencyGraphMiddleware({ rootDir, fileWatcher, onRequest404, onRerunSessions, }) {
    const fileGraph = new dependency_graph_1.DepGraph({ circular: true });
    const urlGraph = new dependency_graph_1.DepGraph({ circular: true });
    let pendingChangedFiles = new Set();
    function getSessionIdsForPath(path) {
        const ids = new Set();
        if (urlGraph.hasNode(path)) {
            for (const dependant of urlGraph.dependantsOf(path)) {
                if (dependant.startsWith('\0')) {
                    const url = new URL(dependant.substring(1));
                    const id = url.searchParams.get(constants_1.PARAM_SESSION_ID);
                    if (!id) {
                        throw new Error('Missing session id parameter');
                    }
                    ids.add(id);
                }
            }
        }
        return ids;
    }
    function getSessionIdsForFile(file) {
        const ids = new Set();
        if (fileGraph.hasNode(file)) {
            for (const dependant of fileGraph.dependantsOf(file)) {
                if (dependant.startsWith('\0')) {
                    const url = new URL(dependant.substring(1));
                    const id = url.searchParams.get(constants_1.PARAM_SESSION_ID);
                    if (!id) {
                        throw new Error('Missing session id parameter');
                    }
                    ids.add(id);
                }
            }
        }
        return ids;
    }
    function syncRerunSessions() {
        const sessionsToRerun = new Set();
        // search dependants of changed files for test HTML files, and reload only those
        for (const file of pendingChangedFiles) {
            for (const id of getSessionIdsForFile(file)) {
                sessionsToRerun.add(id);
            }
        }
        // re run specified sessions
        onRerunSessions(Array.from(sessionsToRerun));
        pendingChangedFiles = new Set();
    }
    const rerunSessions = debounce_1.default(syncRerunSessions, 300);
    function onFileChanged(filePath) {
        pendingChangedFiles.add(filePath);
        rerunSessions();
    }
    fileWatcher.addListener('change', onFileChanged);
    fileWatcher.addListener('unlink', onFileChanged);
    return async (ctx, next) => {
        let dependantUrl;
        let dependencyPath;
        let dependencyUrl;
        if (ctx.path.endsWith('/')) {
            // If the request is for a HTML file without a file extension, we should set itself as the dependant
            dependantUrl = new URL(ctx.href);
            dependencyPath = `${ctx.path}index.html`;
            dependencyUrl = dependencyPath;
        }
        else if (!ctx.headers.referer) {
            // certain files like source maps are fetched without a referer, we skip those
            return next();
        }
        else {
            dependantUrl = new URL(ctx.headers.referer);
            dependencyPath = ctx.path;
            dependencyUrl = ctx.url;
        }
        const dependantFilePath = dependantUrl.searchParams.has(constants_1.PARAM_SESSION_ID)
            ? // the dependant is the test HTML file, we remember the full href and mark it with a null byte
                `\0${dependantUrl.href}`
            : // the dependant is a "regular" file, we resolve it to the file path
                path_1.default.join(rootDir, toFilePath(dependantUrl.pathname));
        const dependencyFilePath = path_1.default.join(rootDir, toFilePath(dependencyPath));
        const dependantPath = dependantUrl.searchParams.has(constants_1.PARAM_SESSION_ID)
            ? dependantFilePath
            : dependantUrl.pathname;
        fileGraph.addNode(dependantFilePath);
        fileGraph.addNode(dependencyFilePath);
        fileGraph.addDependency(dependantFilePath, dependencyFilePath);
        urlGraph.addNode(dependantPath);
        urlGraph.addNode(dependencyUrl);
        urlGraph.addDependency(dependantPath, dependencyUrl);
        await next();
        if (ctx.status === 404) {
            for (const sessionId of getSessionIdsForPath(ctx.url)) {
                onRequest404(sessionId, ctx.url.substring(1));
            }
        }
    };
}
exports.dependencyGraphMiddleware = dependencyGraphMiddleware;
//# sourceMappingURL=dependencyGraphMiddleware.js.map